{#
    basic/layout.html
    ~~~~~~~~~~~~~~~~~

    :copyright: Copyright 2016 by Johannes Schriewer
    :license: BSD, see LICENSE for details.
#}
{%- block doctype -%}
<!DOCTYPE html>
{%- endblock %}

{%- set reldelim1 = reldelim1 is not defined and ' &raquo;' or reldelim1 %}
{%- set reldelim2 = reldelim2 is not defined and ' |' or reldelim2 %}
{%- set render_sidebar = (sidebars != []) %}
{%- set render_parent = (parents != []) %}
{%- set url_root = pathto('', 1) %}
{%- if not embedded and docstitle %}
  {%- set titlesuffix = " &mdash; "|safe + docstitle|e %}
{%- else %}
  {%- set titlesuffix = "" %}
{%- endif %}

{%- macro relbar() %}
    <nav>
      <h3>{{ _('Navigation') }}</h3>
      {%- if render_parent %}
      <ul class="parent">
        {%- for parent in parents %}
          <li>
            <a href="{{ parent.link|e }}">{{ parent.title }}</a>
          </li>
        {%- endfor %}
      </ul>
      {%- else %}
      <ul class="parent">
          <li>
            <a href="{{ pathto(master_doc) }}">{{ docstitle|e }}</a>
          </li>
      </ul>
      {%- endif %}
      <ul class="rel">
        {%- for rellink in rellinks %}
        <li>
          <a href="{{ pathto(rellink[0]) }}" title="{{ rellink[1]|striptags|e }}">{{ rellink[3] }}</a>
        </li>
        {%- endfor %}
      </ul>
    </nav>
{%- endmacro %}

{%- macro sidebar() %}
      {%- if render_sidebar %}
      <aside>
        {%- block sidebarlogo %}
        {%- if logo %}
          <p class="logo"><a href="{{ pathto(master_doc) }}">
            <img class="logo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>
          </a></p>
        {%- endif %}
        {%- endblock %}
        {%- if sidebars != None %}
          {%- for sidebartemplate in sidebars %}
          <section>
          {%- include sidebartemplate %}
          </section>
          {%- endfor %}
        {%- endif %}
      </aside>
      {%- endif %}
{%- endmacro %}

{%- macro script() %}
    <script type="text/javascript">
      var DOCUMENTATION_OPTIONS = {
        URL_ROOT:    '{{ url_root }}',
        VERSION:     '{{ release|e }}',
        COLLAPSE_INDEX: false,
        FILE_SUFFIX: '{{ '' if no_search_suffix else file_suffix }}',
        HAS_SOURCE:  {{ has_source|lower }}
      };
    </script>
    {%- for scriptfile in script_files %}
    <script type="text/javascript" src="{{ pathto(scriptfile, 1) }}"></script>
    {%- endfor %}
{%- endmacro %}

{%- macro css() %}
    <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css" />
    <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}" type="text/css" />
    {%- for cssfile in css_files %}
    <link rel="stylesheet" href="{{ pathto(cssfile, 1) }}" type="text/css" />
    {%- endfor %}
{%- endmacro %}

<html{% if language is not none %} lang="{{ language }}"{% endif %}>
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset={{ encoding }}" />
    {{ metatags }}
    {%- block htmltitle %}
    <title>{{ title|striptags|e }}{{ titlesuffix }}</title>
    {%- endblock %}
    {{ css() }}
    {{ script() }}
    {%- if favicon %}
    <link rel="shortcut icon" href="{{ pathto('_static/' + favicon, 1) }}"/>
    {%- endif %}
{%- block linktags %}
    {%- if hasdoc('about') %}
    <link rel="author" title="{{ _('About these documents') }}" href="{{ pathto('about') }}" />
    {%- endif %}
    {%- if hasdoc('swift-modindex') %}
    <link rel="index" title="{{ _('Index') }}" href="{{ pathto('swift-modindex') }}" />
    {%- else %}
    {%- if hasdoc('genindex') %}
    <link rel="index" title="{{ _('Index') }}" href="{{ pathto('genindex') }}" />
    {%- endif %}
    {%- endif %}
    {%- if hasdoc('copyright') %}
    <link rel="copyright" title="{{ _('Copyright') }}" href="{{ pathto('copyright') }}" />
    {%- endif %}
    <link rel="top" title="{{ docstitle|e }}" href="{{ pathto(master_doc) }}" />
    {%- if parents %}
    <link rel="up" title="{{ parents[-1].title|striptags|e }}" href="{{ parents[-1].link|e }}" />
    {%- endif %}
    {%- if next %}
    <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}" />
    {%- endif %}
    {%- if prev %}
    <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}" />
    {%- endif %}
{%- endblock %}
{%- block extrahead %} {% endblock %}
  </head>
  <body>
{%- block header %}{% endblock %}



<div class="main_content">
<main>
{%- block relbar %}{{ relbar() }}{% endblock %}
{%- block content %}
  {%- block sidebar1 %} {# possible location for sidebar #} {% endblock %}

  {%- block document %}
    {% block body %} {% endblock %}
  {%- endblock %}

{%- endblock %}
</main>
{%- block sidebar %}{{ sidebar() }}{% endblock %}
</div>

{%- block footer %}
    <footer>
    {%- if show_copyright %}
      {%- if hasdoc('copyright') %}
        {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
      {%- else %}
        {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{% endtrans %}
      {%- endif %}
    {%- endif %}
    {%- if last_updated %}
      {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
    {%- endif %}
    {%- if show_sphinx %}
      {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx-doc.org/">Sphinx</a> {{ sphinx_version }} with the AnarchyTools Template.{% endtrans %}
    {%- endif %}
    </footer>
{%- endblock %}
  </body>
</html>
